/**
 * Check if a value is a selection query or set operation.
 * @param {*} value The value to check.
 * @returns {value is Query}
 */
export function isQuery(value: any): value is Query;
/**
 * Check if a value is a selection query.
 * @param {*} value The value to check.
 * @returns {value is SelectQuery}
 */
export function isSelectQuery(value: any): value is SelectQuery;
/**
 * Check if a value is a describe query.
 * @param {*} value The value to check.
 * @returns {value is DescribeQuery}
 */
export function isDescribeQuery(value: any): value is DescribeQuery;
export class Query extends ExprNode {
    /**
     * Create a new WITH clause with the given CTE queries.
     * @param {...WithExpr} expr The WITH CTE queries.
     * @returns {WithClause}
     */
    static with(...expr: WithExpr[]): WithClause;
    /**
     * Create a new select query with the given SELECT expressions.
     * @param {...SelectExpr} expr The SELECT expressions.
     * @returns {SelectQuery}
     */
    static select(...expr: SelectExpr[]): SelectQuery;
    /**
     * Create a new select query with the given FROM expressions.
     * @param {...FromExpr} expr The FROM expressions.
     * @returns {SelectQuery}
     */
    static from(...expr: FromExpr[]): SelectQuery;
    /**
     * Create a new UNION set operation over the given queries.
     * @param {...Query} queries The queries.
     * @returns {SetOperation}
     */
    static union(...queries: Query[]): SetOperation;
    /**
     * Create a new UNION ALL set operation over the given queries.
     * @param {...Query} queries The queries.
     * @returns {SetOperation}
     */
    static unionAll(...queries: Query[]): SetOperation;
    /**
     * Create a new INTERSECT set operation over the given queries.
     * @param {...Query} queries The queries.
     * @returns {SetOperation}
     */
    static intersect(...queries: Query[]): SetOperation;
    /**
     * Create a new EXCEPT set operation over the given queries.
     * @param {...Query} queries The queries.
     * @returns {SetOperation}
     */
    static except(...queries: Query[]): SetOperation;
    /**
     * Create a new describe query for the given input query.
     * @param {Query} query The query to describe.
     * @returns {DescribeQuery}
     */
    static describe(query: Query): DescribeQuery;
    /**
     * Instantiate a new query.
     */
    constructor(type: any);
    /** @type {WithClauseNode[]} */
    _with: WithClauseNode[];
    /** @type {ExprNode[]} */
    _orderby: ExprNode[];
    /** @type {number} */
    _limit: number;
    /** @type {number} */
    _offset: number;
    /** @type {Query | null} */
    cteFor: Query | null;
    /**
     * Return a list of subqueries.
     * @returns {Query[]}
     */
    get subqueries(): Query[];
    /**
     * Clone this query.
     * @returns {Query}
     */
    clone(): Query;
    /**
     * Add WITH common table expressions (CTEs).
     * @param  {...WithExpr} expr Expressions to add.
     * @returns {this}
     */
    with(...expr: WithExpr[]): this;
    /**
     * Add ORDER BY expressions.
     * @param  {...OrderByExpr} expr Expressions to add.
     * @returns
     */
    orderby(...expr: OrderByExpr[]): this;
    /**
     * Set the query result LIMIT.
     * @param {number} value The limit value.
     * @returns {this}
     */
    limit(value: number): this;
    /**
     * Set the query result OFFSET.
     * @param {number} value The offset value.
     * @returns {this}
     */
    offset(value: number): this;
}
export class SelectQuery extends Query {
    /**
     * Instantiate a new select query.
     */
    constructor();
    /** @type {SelectClauseNode[]} */
    _select: SelectClauseNode[];
    /** @type {FromClauseNode[]} */
    _from: FromClauseNode[];
    /** @type {ExprNode[]} */
    _where: ExprNode[];
    /** @type {SampleClauseNode} */
    _sample: SampleClauseNode;
    /** @type {ExprNode[]} */
    _groupby: ExprNode[];
    /** @type {ExprNode[]} */
    _having: ExprNode[];
    /** @type {WindowClauseNode[]} */
    _window: WindowClauseNode[];
    /** @type {ExprNode[]} */
    _qualify: ExprNode[];
    /**
     * Clone this query.
     * @returns {SelectQuery}
     */
    clone(): SelectQuery;
    /**
     * Add SELECT expressions.
     * @param {...SelectExpr} expr Expressions to add.
     * @returns {this}
     */
    select(...expr: SelectExpr[]): this;
    /**
     * Set SELECT expressions, replacing any prior expressions.
     * @param {...SelectExpr} expr Expressions to add.
     * @returns {this}
     */
    setSelect(...expr: SelectExpr[]): this;
    /**
     * Indicate if this query should retrieve distinct values only.
     * @param {boolean} value The distinct flag
     * @returns {this}
     */
    distinct(value?: boolean): this;
    _distinct: boolean;
    /**
     * Add table FROM expressions.
     * @param  {...FromExpr} expr Expressions to add.
     * @returns {this}
     */
    from(...expr: FromExpr[]): this;
    /**
     * Set FROM expressions, replacing any prior expressions.
     * @param {...FromExpr} expr Expressions to add.
     * @returns {this}
     */
    setFrom(...expr: FromExpr[]): this;
    /**
     * Set SAMPLE settings.
     * @param {number | SampleClauseNode} value Either a sample clause node
     *  or the sample size as either a row count or percentage.
     * @param {SampleMethod} [method] The sampling method
     *  to use.
     * @param {number} [seed] The random seed.
     * @returns {this}
     */
    sample(value: number | SampleClauseNode, method?: SampleMethod, seed?: number): this;
    /**
     * Add WHERE expressions.
     * @param  {...FilterExpr} expr Expressions to add.
     * @returns {this}
     */
    where(...expr: FilterExpr[]): this;
    /**
     * Set WHERE expressions, replacing any prior expressions.
     * @param {...FilterExpr} expr Expressions to add.
     * @returns {this}
     */
    setWhere(...expr: FilterExpr[]): this;
    /**
     * Add GROUP BY expressions.
     * @param  {...GroupByExpr} expr Expressions to add.
     * @returns {this}
     */
    groupby(...expr: GroupByExpr[]): this;
    /**
     * Set GROUP BY expressions, replacing any prior expressions.
     * @param {...GroupByExpr} expr Expressions to add.
     * @returns {this}
     */
    setGroupby(...expr: GroupByExpr[]): this;
    /**
     * Add HAVING expressions.
     * @param {...FilterExpr} expr Expressions to add.
     * @returns {this}
     */
    having(...expr: FilterExpr[]): this;
    /**
     * Add WINDOW definitions.
     * @param  {...any} expr Expressions to add.
     * @returns {this}
     */
    window(...expr: any[]): this;
    /**
     * Add QUALIFY expressions.
     * @param {...FilterExpr} expr Expressions to add.
     * @returns {this}
     */
    qualify(...expr: FilterExpr[]): this;
}
export class DescribeQuery extends SQLNode {
    /**
     * Instantiate a describe query.
     */
    constructor(query: any);
    query: any;
    /**
     * Clone this describe query.
     * @returns {DescribeQuery}
     */
    clone(): DescribeQuery;
}
export class SetOperation extends Query {
    /**
     * Instantiate a new set operation instance.
     * @param {string} op The set operation.
     * @param {Query[]} queries The subqueries.
     */
    constructor(op: string, queries: Query[]);
    /**
     * @type {string}
     * @readonly
     */
    readonly op: string;
    /**
     * @type {Query[]}
     * @readonly
     */
    readonly queries: Query[];
    /**
     * Clone this set operation.
     * @returns {SetOperation}
     */
    clone(): SetOperation;
}
import { ExprNode } from './node.js';
import { WithClauseNode } from './with.js';
import type { WithExpr } from '../types.js';
import type { OrderByExpr } from '../types.js';
declare class WithClause {
    /**
     * Instantiate a new WITH clause instance.
     * @param {...WithExpr} expr The WITH CTE queries.
     */
    constructor(...expr: WithExpr[]);
    _with: WithExpr[];
    /**
     * Create a new select query with the given SELECT expressions.
     * @param {...SelectExpr} expr The SELECT expressions.
     * @returns {SelectQuery}
     */
    select(...expr: SelectExpr[]): SelectQuery;
    /**
     * Create a new select query with the given FROM expressions.
     * @param {...FromExpr} expr The FROM expressions.
     * @returns {SelectQuery}
     */
    from(...expr: FromExpr[]): SelectQuery;
    /**
     * Create a new UNION set operation over the given queries.
     * @param {...Query} queries The queries.
     * @returns {SetOperation}
     */
    union(...queries: Query[]): SetOperation;
    /**
     * Create a new UNION ALL set operation over the given queries.
     * @param {...Query} queries The queries.
     * @returns {SetOperation}
     */
    unionAll(...queries: Query[]): SetOperation;
    /**
     * Create a new INTERSECT set operation over the given queries.
     * @param {...Query} queries The queries.
     * @returns {SetOperation}
     */
    intersect(...queries: Query[]): SetOperation;
    /**
     * Create a new EXCEPT set operation over the given queries.
     * @param {...Query} queries The queries.
     * @returns {SetOperation}
     */
    except(...queries: Query[]): SetOperation;
}
import type { SelectExpr } from '../types.js';
import type { FromExpr } from '../types.js';
import { SelectClauseNode } from './select.js';
import { FromClauseNode } from './from.js';
import { SampleClauseNode } from './sample.js';
import { WindowClauseNode } from './window.js';
import type { SampleMethod } from './sample.js';
import type { FilterExpr } from '../types.js';
import type { GroupByExpr } from '../types.js';
import { SQLNode } from './node.js';
export {};
