/**
 * Compute an arg_max aggregate.
 * @param {ExprValue} y The argument to return.
 * @param {ExprValue} x The expression to maximize.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function argmax(y: ExprValue, x: ExprValue): AggregateNode;
/**
 * Compute an arg_min aggregate.
 * @param {ExprValue} y The argument to return.
 * @param {ExprValue} x The expression to minimize.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function argmin(y: ExprValue, x: ExprValue): AggregateNode;
/**
 * Compute an array aggregation.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function arrayAgg(expr: ExprValue): AggregateNode;
/**
 * Compute an average aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function avg(expr: ExprValue): AggregateNode;
/**
 * Compute a correlation aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function corr(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a count aggregate.
 * @param {ExprValue} [expr] An optional expression
 *  to count. If specified, only non-null expression values are counted.
 *  If omitted, all rows within a group are counted.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function count(expr?: ExprValue): AggregateNode;
/**
 * Compute a sample covariance aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function covariance(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a population covariance aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function covarPop(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute an entropy aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function entropy(expr: ExprValue): AggregateNode;
/**
 * Compute a first aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function first(expr: ExprValue): AggregateNode;
/**
 * Compute a geomean aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function geomean(expr: ExprValue): AggregateNode;
/**
 * Compute a sample kurtosis aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function kurtosis(expr: ExprValue): AggregateNode;
/**
 * Compute a median absolute deviation (MAD) aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function mad(expr: ExprValue): AggregateNode;
/**
 * Compute a maximum aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function max(expr: ExprValue): AggregateNode;
/**
 * Compute a median aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function median(expr: ExprValue): AggregateNode;
/**
 * Compute a minimum aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function min(expr: ExprValue): AggregateNode;
/**
 * Compute a mode aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function mode(expr: ExprValue): AggregateNode;
/**
 * Compute a last aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function last(expr: ExprValue): AggregateNode;
/**
 * Compute a product aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function product(expr: ExprValue): AggregateNode;
/**
 * Compute a quantile aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @param {ExprValue} p The quantile value.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function quantile(expr: ExprValue, p: ExprValue): AggregateNode;
/**
 * Compute a linear regression reg_avgX aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function regrAvgX(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a linear regression reg_avgY aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function regrAvgY(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a linear regression count aggregate.
 * This returns the count of rows where both x and y are non-null.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function regrCount(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a linear regression intercept aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function regrIntercept(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a linear regression R^2 aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function regrR2(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a linear regression regr_sxx aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function regrSXX(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a linear regression regr_sxy aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function regrSXY(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a linear regression regr_syy aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function regrSYY(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a linear regression slope aggregate.
 * @param {ExprValue} x The x expression to aggregate.
 * @param {ExprValue} y The y expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function regrSlope(x: ExprValue, y: ExprValue): AggregateNode;
/**
 * Compute a skewness aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function skewness(expr: ExprValue): AggregateNode;
/**
 * Compute a sample standard deviation aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function stddev(expr: ExprValue): AggregateNode;
/**
 * Compute a population standard deviation aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function stddevPop(expr: ExprValue): AggregateNode;
/**
 * Compute a string aggregation.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function stringAgg(expr: ExprValue): AggregateNode;
/**
 * Compute a sum aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function sum(expr: ExprValue): AggregateNode;
/**
 * Compute a sample variance aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function variance(expr: ExprValue): AggregateNode;
/**
 * Compute a population variance aggregate.
 * @param {ExprValue} expr The expression to aggregate.
 * @returns {AggregateNode} A SQL aggregate function call.
 */
export function varPop(expr: ExprValue): AggregateNode;
import type { ExprValue } from '../types.js';
import type { AggregateNode } from '../ast/aggregate.js';
