/**
 * Logical and (AND) operator.
 * @param {...ExprVarArgs} clauses The input expressions.
 * @returns {AndNode}
 */
export function and(...clauses: ExprVarArgs[]): AndNode<any>;
/**
 * Logical or (OR) operator.
 * @param {...ExprVarArgs} clauses The input expressions.
 * @returns {OrNode}
 */
export function or(...clauses: ExprVarArgs[]): OrNode<any>;
/**
 * Logical not (NOT) operator.
 * @param {ExprValue} expr The expression to negate.
 * @returns {UnaryOpNode}
 */
export function not(expr: ExprValue): UnaryOpNode;
/**
 * Null check (IS NULL) operator.
 * @param {ExprValue} expr The expression to test.
 * @returns {UnaryPosftixOpNode}
 */
export function isNull(expr: ExprValue): UnaryPosftixOpNode;
/**
 * Non-null check (IS NOT NULL) operator.
 * @param {ExprValue} expr The expression to test.
 * @returns {UnaryPosftixOpNode}
 */
export function isNotNull(expr: ExprValue): UnaryPosftixOpNode;
/**
 * Bitwise not (~) operator.
 * @param {ExprValue} expr The input expression.
 * @returns {UnaryOpNode}
 */
export function bitNot(expr: ExprValue): UnaryOpNode;
/**
 * Bitwise and (&) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function bitAnd(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Bitwise or (|) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function bitOr(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Bit shift left (<<) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function bitLeft(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Bit shift right (>>) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function bitRight(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Addition (+) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function add(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Subtraction (-) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function sub(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Multiplication (*) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function mul(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Division (/) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function div(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Integer division (//) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function idiv(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Modulo (%) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function mod(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Exponentiation (**) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function pow(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Equality comparision (=) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function eq(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Non-equality comparision (<>) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function neq(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Less-than comparision (<) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function lt(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Greater-than comparision (>) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function gt(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Less-than or equal comparision (<=) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function lte(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Greater-than or equal comparision (>=) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function gte(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Null-inclusive non-equality (IS DISTINCT FROM) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function isDistinct(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Null-inclusive equality (IS NOT DISTINCT FROM) operator.
 * @param {ExprValue} left The left argument.
 * @param {ExprValue} right The right argument.
 * @returns {BinaryOpNode}
 */
export function isNotDistinct(left: ExprValue, right: ExprValue): BinaryOpNode;
/**
 * Range inclusion (BETWEEN) operator.
 * @param {ExprValue} expr The expression to test.
 * @param {ExprValue[]} extent The range extent.
 * @returns {BetweenOpNode}
 */
export function isBetween(expr: ExprValue, extent: ExprValue[]): BetweenOpNode;
/**
 * Range exclusion (NOT BETWEEN) operator.
 * @param {ExprValue} expr The expression to test.
 * @param {ExprValue[]} extent The range extent.
 * @returns {NotBetweenOpNode}
 */
export function isNotBetween(expr: ExprValue, extent: ExprValue[]): NotBetweenOpNode;
/**
 * Set inclusion (IN) operator.
 * @param {ExprValue} expr The expression to test.
 * @param {ExprValue[]} values The included values.
 * @returns {InOpNode}
 */
export function isIn(expr: ExprValue, values: ExprValue[]): InOpNode;
import type { ExprVarArgs } from '../types.js';
import { AndNode } from '../ast/logical-op.js';
import { OrNode } from '../ast/logical-op.js';
import type { ExprValue } from '../types.js';
import { UnaryOpNode } from '../ast/unary-op.js';
import { UnaryPosftixOpNode } from '../ast/unary-op.js';
import { BinaryOpNode } from '../ast/binary-op.js';
import { BetweenOpNode } from '../ast/between-op.js';
import { NotBetweenOpNode } from '../ast/between-op.js';
import { InOpNode } from '../ast/in-op.js';
