/**
 * @typedef {object} BinOptions
 * @property {number} [step] An exact binning step to use.
 * @property {number} [steps] The desired number of binning steps.
 *  This value is a hint, it does not guarantee an exact number of steps.
 * @property {number} [minstep] A minimum binning step value. No generated
 *  step can be less than this value.
 * @property {boolean} [nice] A boolean flag (default true) indicating if bin
 *  extents should be snapped to "nice" numbers such as multiples of 5 or 10.
 * @property {number} [base] A number indicating the the logarithm base to
 *  use for automatic step size determination. Defaults to base 10.
 */
/**
 * Generate a numeric binning scheme suitable for a histogram.
 * @param {number} min The minimum value of the extent to bin.
 * @param {number} max The maximum value of the extent to bin.
 * @param {BinOptions} options Binning scheme options.
 */
export function binSpec(min: number, max: number, options: BinOptions): {
    min: number;
    max: number;
    steps: number;
};
/**
 * Determine a bin step interval.
 * @param {number} span The span from maximum to minimum value.
 * @param {number} steps The approximate number of desired bins.
 * @param {number} [minstep=0] The minimum acceptable bin step size.
 * @param {number} [logb=Math.LN10] The log base for determining
 *  orders of magnitude for step sizes. Defaults to log base 10
 *  (`Math.LN10`). For example to use log base 2, provide the
 *  argument `Math.LN2` instead.
 * @returns {number} The bin step interval (bin size).
 */
export function binStep(span: number, steps: number, minstep?: number, logb?: number): number;
export type BinOptions = {
    /**
     * An exact binning step to use.
     */
    step?: number;
    /**
     * The desired number of binning steps.
     * This value is a hint, it does not guarantee an exact number of steps.
     */
    steps?: number;
    /**
     * A minimum binning step value. No generated
     * step can be less than this value.
     */
    minstep?: number;
    /**
     * A boolean flag (default true) indicating if bin
     * extents should be snapped to "nice" numbers such as multiples of 5 or 10.
     */
    nice?: boolean;
    /**
     * A number indicating the the logarithm base to
     * use for automatic step size determination. Defaults to base 10.
     */
    base?: number;
};
